/*
* Copyright (c) 京北方信息技术股份有限公司 Corporation 2022 . All rights reserved.
*
*/
package net.northking.cctp.scriptcase.api.comLib;

import cn.gjing.tools.auth.annotation.RequiredPermissions;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import net.northking.cctp.common.http.ResultWrapper;
import net.northking.cctp.scriptcase.api.comLib.service.LibraryBusComTreeApiService;
import net.northking.cctp.scriptcase.dto.comLib.LibraryBusComTreeAddDto;
import net.northking.cctp.scriptcase.dto.comLib.LibraryBusComTreeDetailDto;
import net.northking.cctp.scriptcase.dto.comLib.LibraryBusComTreeQueryDto;
import net.northking.cctp.scriptcase.dto.comLib.LibraryBusComTreeUpdateDto;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import java.util.List;


/**
 * 业务组件库目录树 => Restful 类型 HTTP/HTTPS 接口
 *
 * @author maven-cctp-plugin
 * @since 1.0
 */
@Api(tags = "业务组件库目录树")
@RequestMapping("/v1/LibBusComTrees")
@RestController
public class LibraryBusComTreeRestfulCtrl
{
  /**
   * 日志
   */
  private final static Logger logger = LoggerFactory.getLogger(LibraryBusComTreeRestfulCtrl.class);

  @Autowired
  private LibraryBusComTreeApiService apiService;

  /**
   * Restful接口-post： 新增业务组件库目录树
   *
   * @param inputDto         接口数据传输对象
   * @return 主键
   */
  @ApiOperation(value = "新增：业务组件库目录树")
  @RequiredPermissions("ui-saveLibComTree")
  @PostMapping(
          produces = MediaType.APPLICATION_JSON_UTF8_VALUE,
          consumes = MediaType.APPLICATION_JSON_UTF8_VALUE)
  public ResultWrapper<String> add(
          @RequestBody @Validated LibraryBusComTreeAddDto inputDto)
  {
    ResultWrapper<String> wrapper = new ResultWrapper<>();
    String uuid = apiService.add(inputDto);
    return wrapper.success(uuid);
  }

  /**
   * Restful接口-put： 更新业务组件库目录树
   *
   * @param inputDto         接口数据传输对象
   * @return 修改的记录数量
   */
  @ApiOperation(value = "更新：业务组件库目录树")
  @RequiredPermissions("ui-updateLibComTree")
  @PutMapping(
          produces = MediaType.APPLICATION_JSON_UTF8_VALUE,
          consumes = MediaType.APPLICATION_JSON_UTF8_VALUE)
  public ResultWrapper<Integer> update(
          @RequestBody @Validated LibraryBusComTreeUpdateDto inputDto) throws IllegalAccessException
  {
    ResultWrapper<Integer> wrapper = new ResultWrapper<>();
    Integer count = apiService.updateByPK(inputDto);
    return wrapper.success(count);
  }

  /**
   * Restful接口-delete： 删除业务组件库目录树
   *
   * @param uuid       主键
   * @return 删除的记录数量
   */
  @ApiOperation(value = "删除：业务组件库目录树")
  @RequiredPermissions("ui-businessDeleteItem")
  @DeleteMapping(
          value = "/{uuid}",
          produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
  public ResultWrapper<Integer> deleteByPK(
          @PathVariable("uuid") String uuid)
  {
    ResultWrapper<Integer> wrapper = new ResultWrapper<>();
    Integer count = apiService.deleteByPK(uuid);
    return wrapper.success(count);
  }

  /**
   * 获取业务组件库树子级目录
   *
   * @param inputDto       主键
   * @return 接口数据传输对象
   */
  @ApiOperation(value = "获取业务组件库树子级目录")
  @PostMapping(
          value = "/findChildTreeList",
          produces = MediaType.APPLICATION_JSON_UTF8_VALUE,
          consumes = MediaType.APPLICATION_JSON_UTF8_VALUE)
  public ResultWrapper<List<LibraryBusComTreeDetailDto>> findChildTreeList(
          @RequestBody @Validated LibraryBusComTreeQueryDto inputDto)
  {
    ResultWrapper<List<LibraryBusComTreeDetailDto>> wrapper = new ResultWrapper<>();
    List<LibraryBusComTreeDetailDto> dto = apiService.findChildTreeList(inputDto);
    return wrapper.success(dto);
  }

  /**
   * 模糊搜索目录列表
   *
   * @param inputDto       主键
   * @return 接口数据传输对象
   */
  @ApiOperation(value = "模糊查询所有子目录")
  @PostMapping(
          value = "/findChildTreeListLike",
          produces = MediaType.APPLICATION_JSON_UTF8_VALUE,
          consumes = MediaType.APPLICATION_JSON_UTF8_VALUE)
  public ResultWrapper<List<LibraryBusComTreeDetailDto>> findChildTreeListLike(
          @RequestBody @Validated LibraryBusComTreeQueryDto inputDto)
  {
    ResultWrapper<List<LibraryBusComTreeDetailDto>> wrapper = new ResultWrapper<>();
    List<LibraryBusComTreeDetailDto> dto = apiService.findChildTreeListLike(inputDto);
    return wrapper.success(dto);
  }

}
